package test.buffer;

import javax.annotation.Resource;
import javax.ejb.LocalBean;
import javax.ejb.Stateless;
import javax.jms.Connection;
import javax.jms.ConnectionFactory;
import javax.jms.DeliveryMode;
import javax.jms.JMSException;
import javax.jms.MessageProducer;
import javax.jms.Queue;
import javax.jms.Session;
import javax.jms.TextMessage;
import javax.management.AttributeNotFoundException;
import javax.management.InstanceNotFoundException;
import javax.management.MBeanException;
import javax.management.MBeanServer;
import javax.management.MalformedObjectNameException;
import javax.management.ObjectName;
import javax.management.ReflectionException;

import org.jboss.security.util.MBeanServerLocator;

/**
 * Session Bean implementation class MessageGenerator
 */
@Stateless
@LocalBean
public class MessageGenerator implements MessageGeneratorRemote {

	@Resource(mappedName = "java:/JmsXA")
	private ConnectionFactory connectionFactory;

	@Resource(mappedName = "java:/queue/test")
	private Queue queue;

	public MessageGenerator() {
	}

	@Override
	public void generateMessages(int numOfMessages) throws JMSException {

		if (numOfMessages > 0) {

			Connection connection = connectionFactory.createConnection();

			// Make the session transacted
			Session session = connection.createSession(true, 0);
			MessageProducer producer = session.createProducer(queue);
			producer.setDeliveryMode(DeliveryMode.NON_PERSISTENT);

			for (int i = 0; i < numOfMessages; i++) {

				TextMessage message = session.createTextMessage("Message #" + (i + 1));
				producer.send(message);
			}
		}
	}

	@Override
	public int getMessageNum() throws MalformedObjectNameException, AttributeNotFoundException,
			InstanceNotFoundException, MBeanException, ReflectionException {

		MBeanServer mBeanServer = MBeanServerLocator.locateJBoss();

		Long messagesInQueue = -1L;
		ObjectName serverObjectName = new ObjectName(
				"jboss.as:subsystem=messaging,hornetq-server=default,jms-queue=testQueue");
		for (int i = 0; i < 20 && messagesInQueue < 0; i++) {
			messagesInQueue = (Long) mBeanServer.getAttribute(serverObjectName, "messageCount");
		}
		return (int) messagesInQueue.longValue();
	}
}
