package test.buffer;

import java.util.LinkedList;
import java.util.Properties;

import javax.jms.JMSException;
import javax.jms.Message;
import javax.management.AttributeNotFoundException;
import javax.management.InstanceNotFoundException;
import javax.management.MBeanException;
import javax.management.MalformedObjectNameException;
import javax.management.ReflectionException;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;

import org.hornetq.jms.client.HornetQTextMessage;

public class TestRunner {

	private static Context ctx;
	private static MessageGeneratorRemote messageGenerator;

	/**
	 * @param args
	 * @throws NamingException
	 * @throws JMSException
	 * @throws ReflectionException
	 * @throws MBeanException
	 * @throws InstanceNotFoundException
	 * @throws AttributeNotFoundException
	 * @throws MalformedObjectNameException
	 * @throws InterruptedException
	 */
	public static void main(String[] args) throws NamingException, JMSException,
			MalformedObjectNameException, AttributeNotFoundException, InstanceNotFoundException,
			MBeanException, ReflectionException, InterruptedException {

		TestRunner testRunner = new TestRunner();
		testRunner.executeTest();

	}

	private boolean isRunning = true;;

	private void executeTest() throws NamingException, JMSException, MalformedObjectNameException,
			AttributeNotFoundException, InstanceNotFoundException, MBeanException,
			ReflectionException, InterruptedException {
		messageGenerator = connectToBean();

		System.out.println("Messages In queue before test " + messageCount());

		// Create Messages
		messageGenerator.generateMessages(100);

		System.out.println("Messages In queue after test " + messageCount());

		LinkedList<Consumer> consumers = new LinkedList<TestRunner.Consumer>();

		for (int i = 0; i < 10; i++) {

			Consumer consumer = new Consumer(i + 1, this);
			consumer.start();
			consumers.add(consumer);

		}

		do {
		} while (messageCount() > 0);

		isRunning = false;

		// Wait for all the consumers to stop
		for (Consumer consumer : consumers) {
			if (consumer.isAlive()) {
				consumer.join();
			}
		}

		System.out.println("Test Terminated");

	}

	public int messageCount() throws MalformedObjectNameException, AttributeNotFoundException,
			InstanceNotFoundException, MBeanException, ReflectionException, NamingException {

		int messageNum = messageGenerator.getMessageNum();

		return messageNum;

	}

	private MessageGeneratorRemote connectToBean() throws NamingException {
		Properties jndiProps = new Properties();
		jndiProps.put(Context.INITIAL_CONTEXT_FACTORY,
				"org.jboss.naming.remote.client.InitialContextFactory");
		jndiProps.put(Context.PROVIDER_URL, "remote://localhost:4447");
		jndiProps.put("jboss.naming.client.ejb.context", true);
		jndiProps.put(org.jboss.remoting3.RemotingOptions.MAX_OUTBOUND_CHANNELS, 1000);

		ctx = new InitialContext(jndiProps);

		// lookup
		final String appName = "MessageGeneratorEAR";
		final String moduleName = "MessageGenerator";
		final String beanName = "MessageGenerator";
		final String viewClassName = "test.buffer.MessageGeneratorRemote";

		// let's do the lookup
		String ejbName = appName + "/" + moduleName + "/" + beanName + "!" + viewClassName;
		System.out.println("lookup=" + ejbName);
		MessageGeneratorRemote messageGenerator = (MessageGeneratorRemote) ctx.lookup(ejbName);
		return messageGenerator;
	}

	private class Consumer extends Thread {

		private final int consumerId;
		private final TestRunner testRunner;

		public Consumer(int consumerId, TestRunner testRunner) {
			this.consumerId = consumerId;
			this.testRunner = testRunner;
		}

		@Override
		public void run() {
			System.out.println(consumerId + ": Starting Consumer");

			// Connect to the server
			MessagingHandler messagingHandler = new MessagingHandler("localhost", "5445");

			long numberOfEmptyMessagesSinceLastMission = 0;

			// run until stopped
			try {
				do {
					try {
						Message message = null;
						try {

							message = messagingHandler.getMessage();
						} catch (Exception e1) {

							System.err.println(consumerId + ": Refreshing Connections");

							messagingHandler.refreshConnection();
						}

						// if there was no message to handle
						if (message == null) {
							try {

								// sleep for 10 seconds and try again
								Thread.sleep(10000);
								numberOfEmptyMessagesSinceLastMission++;

							} catch (InterruptedException e) {
								e.printStackTrace();
								break;
							}
						} else {

							// handle the message
							System.out.println(consumerId + ": Hadling Message: "
									+ ((HornetQTextMessage) message).getText());

							// Sleep for 5 seconds to cause delay in handling
							try {

								Thread.sleep(5000);
								// Thread.sleep(300000);
								numberOfEmptyMessagesSinceLastMission++;

							} catch (InterruptedException e) {
								e.printStackTrace();
								break;
							}

							messagingHandler.commit();
							numberOfEmptyMessagesSinceLastMission = 0;
						}

						// check if we need to refresh the connection
						if (numberOfEmptyMessagesSinceLastMission > 60) {

							System.err
									.println(consumerId
											+ ": ; Didn't get any message in the last 10 minutes - refreshing connections");
							numberOfEmptyMessagesSinceLastMission = 0;

							messagingHandler.refreshConnection();
						}
					} catch (Exception e2) {

						e2.printStackTrace();
					}
				} while (testRunner.isRunning());
			} catch (Exception e) {
				e.printStackTrace();
			}

			messagingHandler.cleanConnections();
		}

	}

	public boolean isRunning() {
		return isRunning;
	}
}
